import falcorPathParser from 'falcor-path-syntax';
import { stringify } from 'query-string';
import concat from 'lodash/concat';
import filter from 'lodash/filter';
import get from 'lodash/get';
import head from 'lodash/head';
import isNil from 'lodash/isNil';
import keys from 'lodash/keys';
import map from 'lodash/map';
import omit from 'lodash/omit';
import reduce from 'lodash/reduce';
import values from 'lodash/values';
import camelCase from 'lodash/camelCase';
import isArray from 'lodash/isArray';
import isEmpty from 'lodash/isEmpty';
import isObject from 'lodash/isObject';
import kebabCase from 'lodash/kebabCase';
import pullAll from 'lodash/pullAll';
import snakeCase from 'lodash/snakeCase';
import uniq from 'lodash/uniq';
import has from 'lodash/has';
import union from 'lodash/union';
import assign from 'lodash/assign';
import _ from 'lodash';
import forEach from 'lodash/forEach';
import includes from 'lodash/includes';
import isError from 'lodash/isError';
import some from 'lodash/some';

var DEFAULT_FROM_INDEX = 0;
var DEFAULT_TO_INDEX = 19;
var MY_PATH_KEYWORD = 'self';

function buildIdPair() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (!options.id && keys(options).length === 0) {
    return { idKey: 'id' };
  }

  var whitelistedKeys = ['properties', 'andProperties', 'notProperties'];

  var sanitizedOptions = omit(options, whitelistedKeys);

  var idKey = void 0,
      idValue = void 0;
  if (options.id) {
    idKey = 'id';
    idValue = options.id;
  } else {
    idKey = head(keys(sanitizedOptions));
    idValue = options[idKey];
  }

  return { idKey: idKey, idValue: idValue };
}

function buildPaging(options) {
  return {
    from: get(options, 'paging.from', DEFAULT_FROM_INDEX),
    to: get(options, 'paging.to', DEFAULT_TO_INDEX)
  };
}

function buildProperties(defaults) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var properties = concat([], options.properties || defaults);

  var toAnd = options.andProperties;
  if (isArray(toAnd) && !isEmpty(toAnd)) {
    properties = concat(properties, toAnd);
  }

  var toNot = options.notProperties;
  if (isArray(toNot) && !isEmpty(toNot)) {
    properties = pullAll(properties, toNot);
  }

  properties = uniq(properties);

  return properties;
}

function entityNameToFilename(name) {
  return kebabCase(name) + '.js';
}

function entityToAPIEnvelopeKey(entity) {
  return snakeCase(entity.name);
}

function entityToFalcorListLeaf(entity) {
  return snakeCase(entity.name) + '_list';
}

function entityToFalcorMembershipsListLeaf(entity) {
  return 'membership_' + entityToFalcorListLeaf(entity);
}

function entityToFalcorMyListLeaf(entity) {
  return 'my_' + entityToFalcorListLeaf(entity);
}

function entityToFalcorRootLeaf(entity) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var _buildIdPair = buildIdPair(options),
      idKey = _buildIdPair.idKey;

  return idKey === 'id' ? snakeCase(entity.pluralName) : snakeCase(entity.pluralName) + '_by_' + idKey;
}





function normalizeEntityName(entityName) {
  return snakeCase(entityName);
}

function normalizeOptions(options, entity) {
  // Allow camelCase parent names
  if (isObject(options.parent)) {
    var originalName = head(keys(options.parent));
    var snakeCasedName = snakeCase(originalName);

    if (originalName !== snakeCasedName) {
      options.parent[snakeCasedName] = options.parent[originalName];
      delete options.parent[originalName];
    }
  }

  // Allow camelCase payload names
  var camelCasedName = camelCase(entity.name);
  if (
  // Single word entities like "post" will remain "post" when camelCased.
  // Only normalize true camelCase, multi-word entities.
  isObject(options[camelCasedName]) && !isObject(options[entity.name])) {
    options[entity.name] = options[camelCasedName];
    delete options[camelCasedName];
  }
}

function optionsHasQuery(options) {
  return keys(options.query).length > 0;
}

function procedureToFalcorLeaf(procedure) {
  return snakeCase(procedure);
}

function typedEntityToFalcorListPath(entity, options) {
  return [snakeCase(options.type), snakeCase(entity.name) + '_list'];
}

function typedEntityToFalcorMyListPath(entity, options) {
  return ['my_' + snakeCase(options.type), snakeCase(entity.name) + '_list'];
}

var entitySrc = 'entities';

function entityIsUndefinedError(entityName) {
  var filename = entityNameToFilename(entityName);
  return new Error('The ' + entityName + ' entity is undefined. Ensure ' + entitySrc + '/' + filename + ' exists and is exported by ' + entitySrc + '/_catalog.js');
}

function optionsMustHaveError(property) {
  return new Error('options.' + property + ' is required');
}

function optionsMustHaveIdentifierError() {
  return new Error('options must have an id property or an alternate identifier (token, key, etc)');
}

function optionsHas(options, property) {
  return !isNil(get(options, property));
}

function optionsHasPayload(options, entity) {
  var hasName = has(options, entity.name);
  var hasCamelCase = has(options, camelCase(entity.name));
  return hasName || hasCamelCase;
}

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();





var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};





var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();













var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var StandardEntity = function () {
  function StandardEntity(entities, options) {
    classCallCheck(this, StandardEntity);

    this.entities = entities;
    this.options = options;
  }

  createClass(StandardEntity, [{
    key: 'getOne',
    value: function getOne() {
      var _buildIdPair = buildIdPair(this.options),
          idValue = _buildIdPair.idValue;

      if (!idValue) {
        return optionsMustHaveIdentifierError();
      }

      var properties = buildProperties(this.defaultProperties, this.options);
      var rootLeaf = entityToFalcorRootLeaf(this, this.options);

      var paths = this._buildPropertyPaths([rootLeaf, idValue], properties);

      return paths;
    }
  }, {
    key: 'extractOne',
    value: function extractOne(jsonGraph) {
      var _buildIdPair2 = buildIdPair(this.options),
          idValue = _buildIdPair2.idValue;

      var rootLeaf = entityToFalcorRootLeaf(this, this.options);
      var extractedEntity = jsonGraph.json[rootLeaf][idValue];

      return extractedEntity;
    }
  }, {
    key: 'getMany',
    value: function getMany() {
      var _buildPaging = buildPaging(this.options),
          from = _buildPaging.from,
          to = _buildPaging.to;

      var properties = buildProperties(this.defaultProperties, this.options);

      var basePath = this._buildManyPath();

      var rangedPath = concat(basePath, { from: from, to: to });
      var paths = this._buildPropertyPaths(rangedPath, properties);

      paths.push(concat(basePath, 'meta'));

      return paths;
    }
  }, {
    key: 'extractMany',
    value: function extractMany(jsonGraph) {
      var listPath = concat('json', this._buildManyPath());

      var entityList = get(jsonGraph, listPath);
      var content = values(omit(entityList, 'meta'));
      var meta = entityList.meta;

      return { content: content, meta: meta };
    }
  }, {
    key: '_buildManyPath',
    value: function _buildManyPath() {
      var _buildParentEntity2 = this._buildParentEntity(),
          _buildParentEntity3 = slicedToArray(_buildParentEntity2, 2),
          parentEntity = _buildParentEntity3[0],
          parentOptions = _buildParentEntity3[1];

      var basePath = [];
      basePath = this._addParentToPath(basePath, parentEntity, parentOptions);

      var leafPath = optionsHas(this.options, 'type') ? typedEntityToFalcorListPath(this, this.options) : entityToFalcorListLeaf(this);

      basePath = concat(basePath, leafPath);
      basePath = this._addScopeToPath(basePath);
      basePath = this._addQueryToPath(basePath);

      return basePath;
    }
  }, {
    key: 'getMy',
    value: function getMy() {
      var _buildPaging2 = buildPaging(this.options),
          from = _buildPaging2.from,
          to = _buildPaging2.to;

      var properties = buildProperties(this.defaultProperties, this.options);

      var basePath = this._buildMyPath();

      var rangedPath = concat(basePath, { from: from, to: to });
      var paths = this._buildPropertyPaths(rangedPath, properties);

      paths.push(concat(basePath, 'meta'));

      return paths;
    }
  }, {
    key: 'extractMy',
    value: function extractMy(jsonGraph) {
      var listPath = concat('json', this._buildMyPath());

      var entityList = get(jsonGraph, listPath);
      var content = values(omit(entityList, 'meta'));
      var meta = entityList.meta;

      return { content: content, meta: meta };
    }
  }, {
    key: '_buildMyPath',
    value: function _buildMyPath() {
      var basePath = [MY_PATH_KEYWORD];

      var leafPath = optionsHas(this.options, 'type') ? typedEntityToFalcorMyListPath(this, this.options) : entityToFalcorMyListLeaf(this);

      basePath = concat(basePath, leafPath);
      basePath = this._addScopeToPath(basePath);
      basePath = this._addQueryToPath(basePath);

      return basePath;
    }
  }, {
    key: 'getMembership',
    value: function getMembership() {
      var _buildPaging3 = buildPaging(this.options),
          from = _buildPaging3.from,
          to = _buildPaging3.to;

      var properties = buildProperties(this.defaultProperties, this.options);

      var basePath = this._buildMebershipPath(this);

      var rangedPath = concat(basePath, { from: from, to: to });
      var paths = this._buildPropertyPaths(rangedPath, properties);

      paths.push(concat(basePath, 'meta'));

      return paths;
    }
  }, {
    key: 'extractMemberships',
    value: function extractMemberships(jsonGraph) {
      var listPath = concat('json', this._buildMebershipPath());

      var entityList = get(jsonGraph, listPath);
      var content = values(omit(entityList, 'meta'));
      var meta = entityList.meta;

      return { content: content, meta: meta };
    }
  }, {
    key: '_buildMebershipPath',
    value: function _buildMebershipPath() {
      var listLeaf = entityToFalcorMembershipsListLeaf(this);

      var basePath = [MY_PATH_KEYWORD, listLeaf];
      basePath = this._addScopeToPath(basePath);
      basePath = this._addQueryToPath(basePath);

      return basePath;
    }
  }, {
    key: 'find',
    value: function find() {
      var properties = buildProperties(this.defaultProperties, this.options);

      var findPath = this._buildFindPath();

      var paths = this._buildPropertyPaths(findPath, properties);

      return paths;
    }
  }, {
    key: 'extractFind',
    value: function extractFind(jsonGraph) {
      var findPath = this._buildFindPath();

      var content = get(jsonGraph, concat('json', findPath));

      return { content: content };
    }
  }, {
    key: '_buildFindPath',
    value: function _buildFindPath() {
      var _buildParentEntity4 = this._buildParentEntity(),
          _buildParentEntity5 = slicedToArray(_buildParentEntity4, 2),
          parentEntity = _buildParentEntity5[0],
          parentOptions = _buildParentEntity5[1];

      var basePath = [];
      basePath = this._addParentToPath(basePath, parentEntity, parentOptions);

      var leafPath = optionsHas(this.options, 'type') ? typedEntityToFalcorListPath(this, this.options) : entityToFalcorListLeaf(this);

      basePath = concat(basePath, leafPath);
      basePath = this._addScopeToPath(basePath);

      var findPath = concat(basePath, 'find');
      findPath = this._addQueryToPath(findPath);
      findPath = concat(findPath, 'value');

      return findPath;
    }
  }, {
    key: 'findMy',
    value: function findMy() {
      var properties = buildProperties(this.defaultProperties, this.options);

      var findPath = this._buildFindMyPath();

      var paths = this._buildPropertyPaths(findPath, properties);

      return paths;
    }
  }, {
    key: 'extractFindMy',
    value: function extractFindMy(jsonGraph) {
      var findPath = this._buildFindMyPath();

      var content = get(jsonGraph, concat('json', findPath));

      return { content: content };
    }
  }, {
    key: '_buildFindMyPath',
    value: function _buildFindMyPath() {
      var basePath = [MY_PATH_KEYWORD];

      var leafPath = optionsHas(this.options, 'type') ? typedEntityToFalcorMyListPath(this, this.options) : entityToFalcorMyListLeaf(this);

      basePath = concat(basePath, leafPath);
      basePath = this._addScopeToPath(basePath);

      var findPath = concat(basePath, 'find');
      findPath = this._addQueryToPath(findPath);
      findPath = concat(findPath, 'value');

      return findPath;
    }
  }, {
    key: 'create',
    value: function create() {
      if (!optionsHasPayload(this.options, this)) {
        return optionsMustHaveError(this.name);
      }

      // TODO: this one is a bit different, see if we can merge the entityIsUndefinedError stuff
      // globally when parents are extracted
      var parentEntity = void 0,
          parentOptions = void 0;
      if (!isNil(this.options.parent)) {
        var parentEntityName = head(keys(this.options.parent));
        parentEntity = this.entities[parentEntityName];
        parentOptions = this.options.parent[parentEntityName];

        if (isNil(parentEntity)) {
          return entityIsUndefinedError(parentEntityName);
        }

        if (!optionsHas(parentOptions, 'id')) {
          return optionsMustHaveError('parent.' + parentEntityName + '.id');
        }
      }

      var path = void 0;
      if (parentEntity) {
        var rootLeaf = entityToFalcorRootLeaf(parentEntity);
        var listPath = optionsHas(this.options, 'type') ? typedEntityToFalcorListPath(this, this.options) : entityToFalcorListLeaf(this);

        path = concat(rootLeaf, parentOptions.id, listPath, 'add');
      } else {
        var _rootLeaf = entityToFalcorRootLeaf(this);

        path = [_rootLeaf, 'create'];
      }

      var envelopeKey = entityToAPIEnvelopeKey(this);
      var args = [defineProperty({}, envelopeKey, this.options[this.name])];

      return { path: path, args: args };
    }
  }, {
    key: 'createMy',
    value: function createMy() {
      if (!optionsHasPayload(this.options, this)) {
        return optionsMustHaveError(this.name);
      }

      var basePath = [MY_PATH_KEYWORD];

      var listPath = optionsHas(this.options, 'type') ? typedEntityToFalcorMyListPath(this, this.options) : entityToFalcorMyListLeaf(this);

      var path = concat(basePath, listPath, 'add');

      var envelopeKey = entityToAPIEnvelopeKey(this);
      var args = [defineProperty({}, envelopeKey, this.options[this.name])];

      return { path: path, args: args };
    }
  }, {
    key: 'extractCreated',
    value: function extractCreated(jsonGraph) {
      var rootLeaf = entityToFalcorRootLeaf(this);
      var entities = jsonGraph.json[rootLeaf];
      var first = head(keys(entities));
      var extractedEntity = entities[first];

      return extractedEntity;
    }
  }, {
    key: 'invalidateAfterCreate',
    value: function invalidateAfterCreate() {
      if (!isNil(this.options.parent)) {
        var parentEntityName = head(keys(this.options.parent));
        var parentEntity = this.entities[parentEntityName];
        var parentId = this.options.parent[parentEntityName].id;

        var rootLeaf = entityToFalcorRootLeaf(parentEntity);
        var listPath = optionsHas(this.options, 'type') ? typedEntityToFalcorListPath(this, this.options) : entityToFalcorListLeaf(this);

        var paths = [concat(rootLeaf, parentId, listPath), concat(rootLeaf, parentId, listPath, 'length'), concat(rootLeaf, parentId, listPath, 'meta')];

        return paths;
      }
    }
  }, {
    key: 'invalidateAfterCreateMy',
    value: function invalidateAfterCreateMy() {
      var basePath = [MY_PATH_KEYWORD];

      var listPath = optionsHas(this.options, 'type') ? typedEntityToFalcorMyListPath(this, this.options) : entityToFalcorMyListLeaf(this);

      var paths = [concat(basePath, listPath), concat(basePath, listPath, 'length'), concat(basePath, listPath, 'meta')];

      return paths;
    }
  }, {
    key: 'update',
    value: function update() {
      var _this = this;

      if (!optionsHas(this.options, 'id')) {
        return optionsMustHaveError('id');
      }

      if (keys(this.options).length === 1) {
        return new Error('options must include at least one property to update');
      }

      var properties = filter(keys(this.options), function (key) {
        return key !== 'id';
      });
      var rootLeaf = entityToFalcorRootLeaf(this);

      var paths = map(properties, function (property) {
        var path = {
          path: [rootLeaf, _this.options.id, property],
          value: _this.options[property]
        };

        return path;
      });

      return paths;
    }
  }, {
    key: 'invalidateAfterUpdate',
    value: function invalidateAfterUpdate() {
      var rootLeaf = entityToFalcorRootLeaf(this);

      return [[rootLeaf, this.options.id]];
    }
  }, {
    key: 'extractUpdated',
    value: function extractUpdated(jsonGraph) {
      var rootLeaf = entityToFalcorRootLeaf(this);
      var updatedProperties = jsonGraph.json[rootLeaf][this.options.id];

      return updatedProperties;
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      if (!optionsHas(this.options, 'id')) {
        return optionsMustHaveError('id');
      }

      var rootLeaf = entityToFalcorRootLeaf(this);
      var destroyPath = [rootLeaf, this.options.id, 'delete'];

      return destroyPath;
    }
  }, {
    key: 'invalidateAfterDestroy',
    value: function invalidateAfterDestroy() {
      var rootEntityLeaf = entityToFalcorRootLeaf(this);
      var entityId = this.options.id;

      var paths = [[rootEntityLeaf, entityId]];

      if (!isNil(this.options.parent)) {
        var parentEntityName = head(keys(this.options.parent));
        var parentEntity = this.entities[parentEntityName];
        var parentId = this.options.parent[parentEntityName].id;

        var rootParentLeaf = entityToFalcorRootLeaf(parentEntity);
        var listLeaf = entityToFalcorListLeaf(this);

        paths.push([rootParentLeaf, parentId, listLeaf], [rootParentLeaf, parentId, listLeaf, 'length'], [rootParentLeaf, parentId, listLeaf, 'meta']);
      }

      return paths;
    }
  }, {
    key: 'refOne',
    value: function refOne() {
      var _buildIdPair3 = buildIdPair(this.options),
          idValue = _buildIdPair3.idValue;

      var rootLeaf = entityToFalcorRootLeaf(this, this.options);

      return [rootLeaf, idValue];
    }
  }, {
    key: 'refMany',
    value: function refMany() {
      return this._buildManyPath();
    }
  }, {
    key: 'refMy',
    value: function refMy() {
      var leafPath = entityToFalcorMyListLeaf(this);

      return [MY_PATH_KEYWORD, leafPath];
    }
  }, {
    key: 'call',
    value: function call(path, procedure) {
      var procedureLeaf = procedureToFalcorLeaf(procedure);
      return concat(path, procedureLeaf);
    }
  }, {
    key: 'extractCall',
    value: function extractCall(jsonGraph, path) {
      return reduce(path, function (response, segment) {
        return response[segment];
      }, jsonGraph.json);
    }
  }, {
    key: '_buildParentEntity',
    value: function _buildParentEntity() {
      if (!isNil(this.options.parent)) {
        var parentEntityName = head(keys(this.options.parent));

        var parentEntity = this.entities[parentEntityName];
        var parentOptions = this.options.parent[parentEntityName];

        return [parentEntity, parentOptions];
      } else {
        return [];
      }
    }
  }, {
    key: '_buildPropertyPaths',
    value: function _buildPropertyPaths(rootLeaf, properties) {
      return map(properties, function (property) {
        var propertyPath = falcorPathParser(property);
        var path = concat(rootLeaf, propertyPath);

        return path;
      });
    }
  }, {
    key: '_addQueryToPath',
    value: function _addQueryToPath(path) {
      if (optionsHasQuery(this.options)) {
        var qs = stringify(this.options.query, { arrayFormat: 'bracket' });
        path = concat(path, qs);
      }

      return path;
    }
  }, {
    key: '_addScopeToPath',
    value: function _addScopeToPath(path) {
      if (optionsHas(this.options, 'scope')) {
        path = concat(path, this.options.scope);
      }

      return path;
    }
  }, {
    key: '_addParentToPath',
    value: function _addParentToPath(path, parentEntity, parentOptions) {
      if (parentEntity) {
        var rootLeaf = entityToFalcorRootLeaf(parentEntity);
        path = concat(path, [rootLeaf, parentOptions.id]);
      }

      return path;
    }
  }, {
    key: 'name',


    // convenience aliases for instance access
    get: function get$$1() {
      return this.constructor.name;
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return this.constructor.pluralName;
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return this.constructor.defaultProperties;
    }
  }], [{
    key: 'name',
    get: function get$$1() {
      throw new TypeError('Not implemented');
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      throw new TypeError('Not implemented');
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      throw new TypeError('Not implemented');
    }
  }]);
  return StandardEntity;
}();

var _class = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'artifact';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'artifacts';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'description', 'detail', 'owner.id', 'artifactable.id', 'artifactable.id_str', 'artifactable_type'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$2 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'assessment';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'assessments';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'title', 'description', 'state', 'step', 'is_nominal'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$4 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'assessment_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'assessment_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'text'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$6 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'audio_category';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'audio_categories';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$8 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'audio_track';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'audio_tracks';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'author', 'duration', 'src'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$10 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'bookmark';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'bookmarks';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'bookmarkable_type', 'content.id', 'content.banner.src', 'content.description', 'content.title', 'content.owner.name', 'content.owner.avatar.src', 'content.owner.icon.src'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$12 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'bookmark_folder';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'bookmark_folders';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'bookmarks_last_modified_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$14 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'brand';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'brands';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'brandable_id', 'brandable_type', 'r_value', 'g_value', 'b_value', 'hex_value', 'color_priority'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$16 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'champion';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'champions';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'description'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$17 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'channel';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'channels';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'champion_id', 'created_at', 'type', 'member_count', 'unread_posts_count', 'last_read_post_id', 'topic', 'invitable_by', 'banner_image_id', 'banner_image', 'owner_id', 'owner_type', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$19 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'client';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'clients';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'client_type', 'enterprise', 'download_link', 'is_whitelabel_client', 'created_at', 'updated_at', 'valid', 'owner_id', 'owner_type', 'onboarding_id', 'default_landing_page_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$21 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'cohort';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'cohorts';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'description'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$23 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'collection';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'collections';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'title', 'description', 'public'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$24 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, [{
    key: 'invalidateAfterCreate',
    value: function invalidateAfterCreate() {
      var parentEntityName = head(keys(this.options.parent));
      var parentEntity = this.entities[parentEntityName];
      var parentId = this.options.parent[parentEntityName].id;

      var rootLeaf = entityToFalcorRootLeaf(parentEntity);

      var paths = [[rootLeaf, parentId, 'comment_list'], [rootLeaf, parentId, 'comment_list', 'meta'], [rootLeaf, parentId, 'comment_count']];

      return paths;
    }
  }], [{
    key: 'name',
    get: function get$$1() {
      return 'comment';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'comments';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'body', 'markup_tokens', 'post', 'owner.id', 'owner.first_name', 'owner.last_name', 'owner.name', 'owner.avatar.src', 'created_at', 'commentable_type', 'commentable_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$26 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'connection';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'connections';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return [
      // Shared User & Champion properties
      'id', 'target.id',
      // User specific
      'target.username', 'target.first_name', 'target.last_name', 'target.avatar.src',
      // Champion specific
      'target.name', 'target.icon.src'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$30 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'tree';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'trees';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'friendly_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$28 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'content_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'content_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return union(['_type'], _class$30.defaultProperties, _class$23.defaultProperties);
    }
  }]);
  return _class;
}(StandardEntity);

var _class$31 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'content_list_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'content_list_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'content_id', 'content_type', 'new', 'position'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$33 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'content_session';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'content_sessions';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'content_id', 'content_type', 'user_id', 'state', 'usage_mode', 'relevance', 'percent_complete', 'progress_completed', 'progress_total', 'viewed'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$35 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'content_session_tracker';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'content_session_trackers';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'content_session_id', 'trackable_id', 'trackable_type', 'trackable_content_session_id', 'completed_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$37 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'count';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'counts';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['name', 'count'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$39 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'direct_message';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'direct_messages';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$41 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, [{
    key: 'getMany',
    value: function getMany() {
      var entity = this._dynamicEntityForDiscoverType();

      return entity.getMany();
    }
  }, {
    key: 'extractMany',
    value: function extractMany(jsonGraph) {
      var entity = this._dynamicEntityForDiscoverType();

      return entity.extractMany(jsonGraph);
    }

    // discover content is a bit of a hack and redefines the entity's name/pluralName
    // based upon the provided options.  the code below is defining a dynamic entity
    // with the appropriate name/type to allow for path sets that look like:
    //  - discover_content, related_content_list, [0..19], etc.

  }, {
    key: '_dynamicEntityForDiscoverType',
    value: function _dynamicEntityForDiscoverType() {
      var discoverType = this.options.discoverType;
      var defaultProperties = this.defaultProperties.slice(0);

      var TypedEntity = function (_StandardEntity2) {
        inherits(TypedEntity, _StandardEntity2);

        function TypedEntity() {
          classCallCheck(this, TypedEntity);
          return possibleConstructorReturn(this, (TypedEntity.__proto__ || Object.getPrototypeOf(TypedEntity)).apply(this, arguments));
        }

        createClass(TypedEntity, null, [{
          key: 'name',
          get: function get$$1() {
            return discoverType;
          }
        }, {
          key: 'pluralName',
          get: function get$$1() {
            return discoverType;
          }
        }, {
          key: 'defaultProperties',
          get: function get$$1() {
            return defaultProperties;
          }
        }]);
        return TypedEntity;
      }(StandardEntity);

      return new TypedEntity(this.entities, assign({}, this.options, { type: 'discover_content' }));
    }
  }], [{
    key: 'name',
    get: function get$$1() {
      return 'discover_content';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'discover_content';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'content_type', 'content.id', 'content.banner.src', 'content.description', 'content.title', 'content.owner.id', 'content.owner.name', 'content.owner.avatar.src', 'content.owner.icon.src', 'rating'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$43 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'domain';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'domains';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'id_str', 'name', 'description', 'assessment_id', 'assessment_id_str', 'display_score_type', 'preferred_visualization', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$45 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'domain_score';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'domain_scores';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'id_str', 'domain_id', 'domain_id_str', 'content_session_id', 'content_session_id_str', 'score', 'sum_score', 'mean_score', 'normalized_score', 'result_range_id', 'result_range_id_str', 'label', 'description', 'element_id', 'element_id_str', 'created_at', 'updated_at', 'user_id', 'user_id_str', 'score_type'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$47 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'element';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'elements';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'type', 'title', 'subtitle', 'description', 'position'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$49 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'external_application';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'external_applications';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'icon_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$51 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'external_content';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'external_contents';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'title', 'url'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$53 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'feature';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'features';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'namespace', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$55 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'follow';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'follows';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'follower_id', 'follower_type', 'followable_id', 'followable_type'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$57 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'form_response';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'form_responses';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'key', 'owner_type', 'owner_id', 'origin_type', 'origin_id', 'prompt', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$59 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'graph';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'graphs';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['name', 'id', 'data'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$61 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'growth_action';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'growth_actions';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'agent.id', 'agent.name', 'growee.id', 'growee.name', 'growth_relationship.id', 'growth_relationship.owner.id', 'growth_relationship_id', 'state', 'title', 'description', 'due_at', 'completed_at', 'progress_current', 'progress_total'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$63 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'growth_agent_prompt';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'growth_agent_prompts';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'title', 'state', 'type', 'display_type', 'status', 'message', 'created_at', 'updated_at', 'action.id', 'action.due_at', 'action.state', 'action.progress_total', 'action.progress_current', 'action.growee.id', 'action.growee.name', 'action.growee.email', 'action.growee.avatar.id', 'action.growee.avatar.src', 'action.agent.id', 'action.agent.name', 'action.relationship.id', 'action.relationship.owner.id', 'action.description', 'action.title'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$65 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'growth_agent_response';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'growth_agent_responses';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'message'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$67 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'growth_relationship';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'growth_relationships';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'owner_type', 'owner.id', 'owner.name', 'growee.id', 'growee.name', 'agent.id', 'agent.name', 'space_id', 'to_do_list.id', 'to_do_list.name', 'show_agent_prompts', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$69 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'growth_relationship_event';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'growth_relationship_events';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'growth_relationship_id', 'agent.id', 'agent.name', 'source_type', 'source_id', 'external_application_id', 'growth_action_id', 'space_id', 'type', 'details', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$71 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'home_champion';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'home_champions';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'user_id', 'champion_id', 'position'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$73 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'industry';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'industries';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$75 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'integration';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'integrations';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'icon_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$77 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'integration_configuration';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'integration_configurations';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'champion_id', 'user_id', 'integration_id', 'type', 'status'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$79 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'invitation';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'invitations';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'user_id', 'onboarding_id', 'email', 'token', 'sent_at', 'accepted_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$81 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'item_type', 'champion_id', 'text', 'identifier', 'is_nominal', 'id_str', 'champion_id_str', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$83 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'item_option';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'item_options';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'item_id', 'value', 'text', 'position', 'created_at', 'updated_at', 'id_str', 'item_id_str'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$85 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'item_response';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'item_responses';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'content_session_id', 'item_id', 'item_option_id', 'response_count'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$87 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'landing_page';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'landing_pages';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'title', 'kind', 'slug', 'banner_url', 'description', 'public', 'banner_id', 'owner_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$89 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'license';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'licenses';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$91 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'licensed_cohort';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'licensed_cohorts';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$93 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'licensed_content_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'licensed_content_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$95 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'link';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'links';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'created_at', 'updated_at', 'shared_in_id', 'shared_in_type', 'shared_by_id', 'content'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$97 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'locale';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'locales';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'code', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$99 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'media';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'media';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'description', 'caption', 'created_at', 'updated_at', 'owner_type', 'owner_id', 'src', 'meta'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$101 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'media_collection';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'media_collections';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'media_collection_type', 'name', 'owner_id', 'owner_type', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$103 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'membership';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'memberships';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'membershipable_id', 'membershipable_type', 'member_id', 'member_type', 'status', 'roles', 'created_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$105 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'membership_invitation';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'membership_invitations';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'status', 'message', 'token', 'inviter_id', 'membership_id', 'invitable_id', 'invitable_type', 'invitee_id', 'invitee_type'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$107 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'note';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'notes';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'body'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$109 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'notification';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'notifications';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'status', 'triggering_user_id', 'created_at', 'updated_at', 'subject_type', 'subject_id', 'persona_id', 'persona_type', 'body', 'title', 'unique_hash', 'push_params', 'payload', 'user_id', 'event_type', 'champion_id', 'event_namespace'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$111 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'notification_setting';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'notification_settings';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'setting_type', 'event_namespace', 'label', 'delivery_channels'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$113 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'oauth_consumer_app';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'oauth_consumer_apps';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$115 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'onboarding';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'onboardings';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'title', 'message', 'invite_code', 'display_downloads', 'branch_link'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$117 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, [{
    key: 'getMany',
    value: function getMany() {
      var basePath = this._buildManyPath();

      return [_.concat(basePath, this.options.option)];
    }
  }], [{
    key: 'name',
    get: function get$$1() {
      return 'option_value';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'option_values';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['label', 'value'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$119 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'organization';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'organizations';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$121 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'permission';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'permissions';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'i18n_key', 'key'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$123 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'plan';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'plans';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$125 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'post';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'posts';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'body', 'artifact_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$127 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'progress_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'progress_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'action', 'title', 'description', 'status', 'completed', 'hidden', 'assigner_type', 'linkable_type', 'updated_at', 'created_at', 'date_due'];
    }
  }]);
  return _class;
}(StandardEntity);

var defaultProperties = _class$16.defaultProperties;

var _class$129 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'recommended_champion';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'recommended_champions';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return defaultProperties;
    }
  }]);
  return _class;
}(StandardEntity);

var _class$131 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'result';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'results';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'title', 'description', 'evaluate', 'assessment_id', 'image_id', 'created_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$133 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'result_range';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'result_ranges';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'id_str', 'updated_at', 'name', 'description', 'range_start', 'range_end', 'rangeable_id', // converto to polymorphic relation
      'rangeable_type', // when scales and domain routes are available
      'media_id', 'media_id_str'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$135 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'review';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'reviews';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'rating'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$137 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'role';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'roles';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'key', 'i18n_key', 'title', 'description', 'role_group_name', 'role_group_default', 'position', 'role_group_description'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$139 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'role_assignment';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'role_assignments';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'champion_id', 'user_id', 'role_id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$141 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'rss_feed';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'rss_feeds';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'champion_id', 'feed_type', 'title', 'url', 'last_post_created', 'enabled', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$143 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'rss_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'rss_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'rss_feed_id', 'title', 'summary', 'guid', 'image', 'link', 'published_date', 'enclosure_url', 'event_date', 'feed_type', 'feed_title', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$145 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'scale';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'scales';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'scale_type', 'parent_scale_id', 'user_id', 'name', 'description', 'created_at', 'updated_at', 'champion_id', 'display_score_type', 'identifier', 'parent_scale_id_str', 'title', 'id_str', 'user_id_str', 'champion_id_str', 'max_score', 'min_score', 'max_sum_score', 'max_mean_score', 'min_sum_score', 'min_mean_score'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$147 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'scale_score';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'scale_scores';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'id_str', 'scale_id', 'scale_id_str', 'content_session_id', 'content_session_id_str', 'score', 'sum_score', 'mean_score', 'normalized_score', 'result_range_id', 'result_range_id_str', 'label', 'description', 'element_id', 'element_id_str', 'created_at', 'updated_at', 'user_id', 'user_id_str'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$149 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'social_profile_link';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'social_profile_links';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'link_type', 'link'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$151 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'space';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'spaces';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'topic', 'champion_id', 'type'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$153 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'system_prompt_view';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'system_prompt_views';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'key'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$155 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'timeline';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'timelines';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$157 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'to_do_item';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'to_do_items';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'title', 'description', 'action', 'date_due'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$159 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'to_do_list';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'to_do_lists';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'created_at', 'updated_at'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$161 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'topic';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'topics';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$163 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'tour_tip_view';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'tour_tip_views';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'state'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$165 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'user';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'users';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'username', 'name'];
    }
  }]);
  return _class;
}(StandardEntity);

var _class$167 = function (_StandardEntity) {
  inherits(_class, _StandardEntity);

  function _class() {
    classCallCheck(this, _class);
    return possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).apply(this, arguments));
  }

  createClass(_class, null, [{
    key: 'name',
    get: function get$$1() {
      return 'user_to_do_list';
    }
  }, {
    key: 'pluralName',
    get: function get$$1() {
      return 'user_to_do_lists';
    }
  }, {
    key: 'defaultProperties',
    get: function get$$1() {
      return ['id', 'name', 'public', 'item_count', 'completed_item_count'];
    }
  }]);
  return _class;
}(StandardEntity);



var entities = Object.freeze({
	artifact: _class,
	artifacts: _class,
	assessment: _class$2,
	assessments: _class$2,
	assessment_item: _class$4,
	assessment_items: _class$4,
	audio_category: _class$6,
	audio_categories: _class$6,
	audio_track: _class$8,
	audio_tracks: _class$8,
	bookmark: _class$10,
	bookmarks: _class$10,
	bookmark_folder: _class$12,
	bookmark_folders: _class$12,
	brand: _class$14,
	brands: _class$14,
	champion: _class$16,
	champions: _class$16,
	channel: _class$17,
	channels: _class$17,
	client: _class$19,
	clients: _class$19,
	cohort: _class$21,
	cohorts: _class$21,
	collection: _class$23,
	collections: _class$23,
	comment: _class$24,
	comments: _class$24,
	connection: _class$26,
	connections: _class$26,
	content_item: _class$28,
	content_items: _class$28,
	content_list_item: _class$31,
	content_list_items: _class$31,
	content_session: _class$33,
	content_sessions: _class$33,
	content_session_tracker: _class$35,
	content_session_trackers: _class$35,
	count: _class$37,
	counts: _class$37,
	direct_message: _class$39,
	direct_messages: _class$39,
	discover_content: _class$41,
	domain: _class$43,
	domains: _class$43,
	domain_score: _class$45,
	domain_scores: _class$45,
	element: _class$47,
	elements: _class$47,
	external_application: _class$49,
	external_applications: _class$49,
	external_content: _class$51,
	external_contents: _class$51,
	feature: _class$53,
	features: _class$53,
	follow: _class$55,
	follows: _class$55,
	form_response: _class$57,
	form_responses: _class$57,
	graph: _class$59,
	graphs: _class$59,
	growth_action: _class$61,
	growth_actions: _class$61,
	growth_agent_prompt: _class$63,
	growth_agent_prompts: _class$63,
	growth_agent_response: _class$65,
	growth_agent_responses: _class$65,
	growth_relationship: _class$67,
	growth_relationships: _class$67,
	growth_relationship_event: _class$69,
	growth_relationship_events: _class$69,
	home_champion: _class$71,
	home_champions: _class$71,
	industry: _class$73,
	industries: _class$73,
	integration: _class$75,
	integrations: _class$75,
	integration_configuration: _class$77,
	integration_configurations: _class$77,
	invitation: _class$79,
	invitations: _class$79,
	item: _class$81,
	items: _class$81,
	item_option: _class$83,
	item_options: _class$83,
	item_response: _class$85,
	item_responses: _class$85,
	landing_page: _class$87,
	landing_pages: _class$87,
	license: _class$89,
	licenses: _class$89,
	licensed_cohort: _class$91,
	licensed_cohorts: _class$91,
	licensed_content_item: _class$93,
	licensed_content_items: _class$93,
	link: _class$95,
	links: _class$95,
	locale: _class$97,
	locales: _class$97,
	media: _class$99,
	media_collection: _class$101,
	media_collections: _class$101,
	membership: _class$103,
	memberships: _class$103,
	membership_invitation: _class$105,
	membership_invitations: _class$105,
	note: _class$107,
	notes: _class$107,
	notification: _class$109,
	notifications: _class$109,
	notification_setting: _class$111,
	notification_settings: _class$111,
	oauth_consumer_app: _class$113,
	oauth_consumer_apps: _class$113,
	onboarding: _class$115,
	onboardings: _class$115,
	option_value: _class$117,
	option_values: _class$117,
	organization: _class$119,
	organizations: _class$119,
	permission: _class$121,
	permissions: _class$121,
	plan: _class$123,
	plans: _class$123,
	post: _class$125,
	posts: _class$125,
	progress_item: _class$127,
	progress_items: _class$127,
	recommended_champion: _class$129,
	recommended_champions: _class$129,
	result: _class$131,
	results: _class$131,
	result_range: _class$133,
	result_ranges: _class$133,
	review: _class$135,
	reviews: _class$135,
	role: _class$137,
	roles: _class$137,
	role_assignment: _class$139,
	role_assignments: _class$139,
	rss_feed: _class$141,
	rss_feeds: _class$141,
	rss_item: _class$143,
	rss_items: _class$143,
	scale: _class$145,
	scales: _class$145,
	scale_score: _class$147,
	scale_scores: _class$147,
	social_profile_link: _class$149,
	social_profile_links: _class$149,
	space: _class$151,
	spaces: _class$151,
	system_prompt_view: _class$153,
	system_prompt_views: _class$153,
	timeline: _class$155,
	timelines: _class$155,
	to_do_item: _class$157,
	to_do_items: _class$157,
	to_do_list: _class$159,
	to_do_lists: _class$159,
	topic: _class$161,
	topics: _class$161,
	tour_tip_view: _class$163,
	tour_tip_views: _class$163,
	tree: _class$30,
	trees: _class$30,
	user: _class$165,
	users: _class$165,
	user_to_do_list: _class$167,
	user_to_do_lists: _class$167
});

function buildEntity(entityName, options) {
  var Entity = entities[normalizeEntityName(entityName)];

  if (Entity) {
    normalizeOptions(options, Entity);
    return new Entity(entities, options);
  }
}

var SuperGloo = function () {
  function SuperGloo() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    classCallCheck(this, SuperGloo);

    var requiredProperties = ['falcorModel'];

    if (some(requiredProperties, function (_$$1) {
      return isNil(get(config, _$$1));
    })) {
      throw new Error('One of the following required config parameters is undefined: ' + requiredProperties.join(', '));
    }

    this.falcor = config.falcorModel;
  }

  /*
   * Standard interfaces
   */

  createClass(SuperGloo, [{
    key: 'setModel',
    value: function setModel(model) {
      if (isNil(model)) {
        throw new Error('When setting the model, the model cannot be null or undefined');
      }

      this.falcor = model;
    }
  }, {
    key: 'getOne',
    value: function getOne(entityName) {
      var _falcor,
          _this = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.getOne();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor = this.falcor).get.apply(_falcor, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedEntity = void 0;
        if (isNil(jsonGraph)) {
          extractedEntity = null;
        } else {
          extractedEntity = entity.extractOne(jsonGraph);
        }

        return extractedEntity;
      }).catch(function (response) {
        return _this.handleError({ action: 'getOne', entity: entity, response: response });
      });
    }
  }, {
    key: 'getMany',
    value: function getMany(entityName) {
      var _falcor2,
          _this2 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.getMany();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor2 = this.falcor).get.apply(_falcor2, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedResponse = void 0;
        if (isNil(jsonGraph)) {
          extractedResponse = null;
        } else {
          extractedResponse = entity.extractMany(jsonGraph);
        }

        return extractedResponse;
      }).catch(function (response) {
        return _this2.handleError({ action: 'getMany', entity: entity, response: response });
      });
    }
  }, {
    key: 'getMy',
    value: function getMy(entityName) {
      var _falcor3,
          _this3 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.getMy();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor3 = this.falcor).get.apply(_falcor3, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedResponse = void 0;
        if (isNil(jsonGraph)) {
          extractedResponse = null;
        } else {
          extractedResponse = entity.extractMy(jsonGraph);
        }

        return extractedResponse;
      }).catch(function (response) {
        return _this3.handleError({ action: 'getMy', entity: entity, response: response });
      });
    }
  }, {
    key: 'getMembership',
    value: function getMembership(entityName) {
      var _falcor4,
          _this4 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.getMembership();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor4 = this.falcor).get.apply(_falcor4, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedResponse = void 0;
        if (isNil(jsonGraph)) {
          extractedResponse = null;
        } else {
          extractedResponse = entity.extractMemberships(jsonGraph);
        }

        return extractedResponse;
      }).catch(function (response) {
        return _this4.handleError({ action: 'getMembership', entity: entity, response: response });
      });
    }
  }, {
    key: 'getMe',
    value: function getMe() {
      var _falcor5,
          _this5 = this;

      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var entity = buildEntity('user', options);
      var defaultProperties = entity.defaultProperties;


      var properties = buildProperties(defaultProperties, options);
      var paths = map(properties, function (property) {
        var propertyPath = falcorPathParser(property);
        var path = concat(MY_PATH_KEYWORD, propertyPath);

        return path;
      });

      return (_falcor5 = this.falcor).get.apply(_falcor5, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedResponse = !isNil(jsonGraph) ? jsonGraph.json[MY_PATH_KEYWORD] : null;

        return extractedResponse;
      }).catch(function (response) {
        return _this5.handleError({ action: 'getMe', response: response });
      });
    }
  }, {
    key: 'find',
    value: function find(entityName) {
      var _falcor6,
          _this6 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.find();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor6 = this.falcor).get.apply(_falcor6, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedEntity = void 0;
        if (isNil(jsonGraph)) {
          extractedEntity = null;
        } else {
          extractedEntity = entity.extractFind(jsonGraph);
        }

        return extractedEntity;
      }).catch(function (response) {
        return _this6.handleError({ action: 'find', entity: entity, response: response });
      });
    }
  }, {
    key: 'findMy',
    value: function findMy(entityName) {
      var _falcor7,
          _this7 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.findMy();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor7 = this.falcor).get.apply(_falcor7, toConsumableArray(paths)).then(function (jsonGraph) {
        var extractedEntity = void 0;
        if (isNil(jsonGraph)) {
          extractedEntity = null;
        } else {
          extractedEntity = entity.extractFindMy(jsonGraph);
        }

        return extractedEntity;
      }).catch(function (response) {
        return _this7.handleError({ action: 'findMy', entity: entity, response: response });
      });
    }
  }, {
    key: 'create',
    value: function create(entityName) {
      var _this8 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var callOptions = entity.create();

      if (isError(callOptions)) {
        return Promise.reject(callOptions);
      }

      var path = callOptions.path,
          args = callOptions.args;


      return this.falcor.call(path, args).then(function (jsonGraph) {
        var extractedEntity = void 0;
        if (isNil(jsonGraph)) {
          extractedEntity = null;
        } else {
          extractedEntity = entity.extractCreated(jsonGraph);
        }

        var pathsToInvalidate = entity.invalidateAfterCreate();

        if (!isEmpty(pathsToInvalidate)) {
          forEach(pathsToInvalidate, function (path) {
            _this8.falcor.invalidate(path);
          });
        }

        return extractedEntity;
      }).catch(function (response) {
        return _this8.handleError({ action: 'create', entity: entity, response: response });
      });
    }
  }, {
    key: 'createMy',
    value: function createMy(entityName) {
      var _this9 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var callOptions = entity.createMy();

      if (isError(callOptions)) {
        return Promise.reject(callOptions);
      }

      var path = callOptions.path,
          args = callOptions.args;


      return this.falcor.call(path, args).then(function (jsonGraph) {
        var extractedEntity = void 0;
        if (isNil(jsonGraph)) {
          extractedEntity = null;
        } else {
          extractedEntity = entity.extractCreated(jsonGraph);
        }

        var pathsToInvalidate = entity.invalidateAfterCreateMy();

        if (!isEmpty(pathsToInvalidate)) {
          forEach(pathsToInvalidate, function (path) {
            _this9.falcor.invalidate(path);
          });
        }

        return extractedEntity;
      }).catch(function (response) {
        return _this9.handleError({ action: 'createMy', entity: entity, response: response });
      });
    }
  }, {
    key: 'update',
    value: function update(entityName) {
      var _falcor8,
          _this10 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var paths = entity.update();

      if (isError(paths)) {
        return Promise.reject(paths);
      }

      return (_falcor8 = this.falcor).set.apply(_falcor8, toConsumableArray(paths)).then(function (jsonGraph) {
        var updatedProperties = void 0;
        if (isNil(jsonGraph)) {
          updatedProperties = null;
        } else {
          updatedProperties = entity.extractUpdated(jsonGraph);
        }

        var pathsToInvalidate = entity.invalidateAfterUpdate();

        if (!isEmpty(pathsToInvalidate)) {
          forEach(pathsToInvalidate, function (path) {
            _this10.falcor.invalidate(path);
          });
        }

        return updatedProperties;
      }).catch(function (response) {
        return _this10.handleError({ action: 'update', entity: entity, response: response });
      });
    }
  }, {
    key: 'destroy',
    value: function destroy(entityName) {
      var _this11 = this;

      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      var destroyPath = entity.destroy();

      if (isError(destroyPath)) {
        return Promise.reject(destroyPath);
      }

      return this.falcor.call(destroyPath).then(function () {
        var pathsToInvalidate = entity.invalidateAfterDestroy();

        if (!isEmpty(pathsToInvalidate)) {
          forEach(pathsToInvalidate, function (path) {
            _this11.falcor.invalidate(path);
          });
        }
      }).catch(function (response) {
        return _this11.handleError({ action: 'destroy', entity: entity, response: response });
      });
    }
  }, {
    key: 'refOne',
    value: function refOne(entityName) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      this._entity = entity;
      this._path = entity.refOne();

      return this;
    }
  }, {
    key: 'refMany',
    value: function refMany(entityName) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      this._entity = entity;
      this._path = entity.refMany();

      return this;
    }
  }, {
    key: 'refMy',
    value: function refMy(entityName) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var entity = buildEntity(entityName, options);

      if (isNil(entity)) {
        return Promise.reject(entityIsUndefinedError(entityName));
      }

      this._entity = entity;
      this._path = entity.refMy();

      return this;
    }
  }, {
    key: 'call',
    value: function call(procedure) {
      var _this12 = this;

      var body = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var path = this._entity.call(this._path, procedure);

      return this.falcor.call(path, [body]).then(function (jsonGraph) {
        var extractedResponse = void 0;
        if (isNil(jsonGraph)) {
          extractedResponse = null;
        } else {
          extractedResponse = _this12._entity.extractCall(jsonGraph, path);
        }

        return extractedResponse;
      }).catch(function (response) {
        return _this12.handleError({ action: 'call', entity: _this12._entity, response: response });
      });
    }

    // Currently only works when chained with refOne or refMany. Given refMy
    // depends on the server-resolved 'self' path, it will need a refactor
    // before we can refMy().invalidate().
    //
    // TODO: Refactor refMy, add refMembership

  }, {
    key: 'invalidate',
    value: function invalidate() {
      return Promise.resolve(this.falcor.invalidate([this._path]));
    }
  }, {
    key: 'handleError',
    value: function handleError(_ref) {
      var action = _ref.action,
          entity = _ref.entity,
          response = _ref.response;

      var singularActions = ['getOne', 'create', 'update', 'destroy'];

      var error = void 0;
      if (response instanceof Error) {
        error = response;
      } else {
        var actionDetails = void 0;
        if (!isNil(entity)) {
          var nameProperty = includes(singularActions, action) ? 'name' : 'pluralName';

          actionDetails = action + ' ' + entity[nameProperty];
        } else {
          actionDetails = action;
        }

        var message = isArray(response) && !isEmpty(response) ? JSON.stringify(response[0].value) : 'Unknown error encountered';

        error = new Error('Could not ' + actionDetails + ': ' + message);

        // If we don't invalidate the path, follow on calls to the same path
        // will always resolve to the error that's already in the client cache
        this.falcor.invalidate(response.path);
      }

      return Promise.reject(error);
    }
  }]);
  return SuperGloo;
}();

export default SuperGloo;
